#!/bin/sh
#
# This is simplifing version of certificates generator of Wazuh project for FreeBSD
#

gen::check::req() {
        if [ ! -f "indexer.lst" ] || [ ! -f "server.lst" ] || [ ! -f "dashboard.lst" ]; then
		echo "#########################################################"
		echo "indexer.lst, server.lst or dashboard.lst files not found."
		echo "#########################################################"

		exit;
	fi

	for f in $(echo "indexer.lst server.lst dashboard.lst")
	do
		grep -e "^[^#].*_ip=" ${f}

		if [ $? -gt 0 ]; then
			echo "#################################################"
			echo "${f} is empty or not contains valid entries."
			echo "#################################################"

			exit;
		fi
	done
}

gen::root::ca() {
	echo "#############################"
	echo "Generating RootCa certificate"
	echo "#############################"
	$(openssl req -x509 -new -nodes -newkey rsa:2048 -keyout wazuh-certificates/root-ca.key \
		-out wazuh-certificates/root-ca.pem -batch -subj '/OU=Wazuh/O=Wazuh/L=California/' -days 3650)
}

gen::admin::cert() {
	echo "############################"
        echo "Generating Admin certificate"
        echo "############################"

	$(openssl genrsa -out wazuh-certificates/admin-key-temp.pem 2048)

	$(openssl pkcs8 -inform PEM -outform PEM -in wazuh-certificates/admin-key-temp.pem \
		-topk8 -nocrypt -v1 PBE-SHA1-3DES -out wazuh-certificates/admin-key.pem)

	$(openssl req -new -key wazuh-certificates/admin-key.pem -out wazuh-certificates/admin.csr \
		-batch -subj '/C=US/L=California/O=Wazuh/OU=Wazuh/CN=admin')

	$(openssl x509 -days 3650 -req -in wazuh-certificates/admin.csr -CA wazuh-certificates/root-ca.pem \
		-CAkey wazuh-certificates/root-ca.key -CAcreateserial -sha256 -out wazuh-certificates/admin.pem)
}

gen::cert::config() {
	local _name="$1"
	local _ip="$2"

	echo "###################################"
        echo "Generating ${_name} config file"
        echo "###################################"

	$(cat cert.conf.template | sed -e "s|%%name%%|${_name}|g" -e "s|%%ip%%|${_ip}|g" > "wazuh-certificates/${_name}.conf")
}

gen::cert() {
	local _name="$1"
	local _ip="$2"

	if [ ! -f "wazuh-certificates/${_name}-key.pem" ]; then
		gen::cert::config "${_name}" "${_ip}"

		echo "####################################"
	        echo "Generating ${_name} certificate"
	        echo "####################################"

		$(openssl req -new -nodes -newkey rsa:2048 -keyout wazuh-certificates/${_name}-key.pem \
			-out wazuh-certificates/${_name}.csr -config wazuh-certificates/${_name}.conf)

		$(openssl x509 -req -in wazuh-certificates/${_name}.csr -CA wazuh-certificates/root-ca.pem -CAkey \
			wazuh-certificates/root-ca.key -CAcreateserial -out wazuh-certificates/${_name}.pem \
				-extfile wazuh-certificates/${_name}.conf -extensions v3_req -days 3650)
	else
		echo "####################################"
                echo "Skip. ${_name} certificate exists"
                echo "####################################"
	fi
}

gen::init() {
	local _file
	local _indexer
	local _server
	local _dashboard
	local _name
	local _ip

	gen::check::req

	mkdir -p "wazuh-certificates"

        if [ ! -f "wazuh-certificates/root-ca.key" ]; then
                gen::root::ca
        fi
        if [ ! -f "wazuh-certificates/admin-key.pem" ]; then
                gen::admin::cert
        fi

	_indexer=$(cat indexer.lst | sed -e '/^#/d' | sed '/^$/d')

	for _file in ${_indexer}
	do
		_name=$(echo ${_file} | grep ".*_ip=" | cut -d '_' -f1)
		_ip=$(echo ${_file} | grep ".*_ip=" | cut -d '=' -f2)

		gen::cert "${_name}" "${_ip}"
	done

        _server=$(cat server.lst | sed -e '/^#/d' | sed '/^$/d')

        for _file in ${_server}
        do
                _name=$(echo ${_file} | grep ".*_ip=" | cut -d '_' -f1)
                _ip=$(echo ${_file} | grep ".*_ip=" | cut -d '=' -f2)

		gen::cert "${_name}" "${_ip}"
        done

        _dashboard=$(cat dashboard.lst | sed -e '/^#/d' | sed '/^$/d')

        for _file in ${_dashboard}
        do
                _name=$(echo ${_file} | grep ".*_ip=" | cut -d '_' -f1)
                _ip=$(echo ${_file} | grep ".*_ip=" | cut -d '=' -f2)

		gen::cert "${_name}" "${_ip}"
        done
}

echo "####################################################################################"
echo "Do you want generate root, admin, server, indexer and dashboard certificates? (y/n):"
echo "####################################################################################"

read answer

if [ "${answer}" = "y" ]; then
	gen::init
else
	echo "Bye!"
	exit;
fi
